@@ -4,7 +4,7 @@ package JSON::Schema::Modern::Vocabulary;
 # vim: set ts=8 sts=2 sw=2 tw=100 et :
 # ABSTRACT: Base role for JSON Schema vocabulary classes
 
-our $VERSION = '0.512';
+our $VERSION = '0.513';
 
 use 5.016;
 no if "$]" >= 5.031009, feature => 'indirect';
@@ -19,13 +19,13 @@ requires qw(vocabulary keywords);
 
 sub traverse {
   my ($self, $schema, $state) = @_;
-  $state->{evaluator}->_traverse($schema, $state);
+  $state->{evaluator}->_traverse_subschema($schema, $state);
 }
 
 sub traverse_subschema {
   my ($self, $schema, $state) = @_;
 
-  $state->{evaluator}->_traverse($schema->{$state->{keyword}},
+  $state->{evaluator}->_traverse_subschema($schema->{$state->{keyword}},
     +{ %$state, schema_path => $state->{schema_path}.'/'.$state->{keyword} });
 }
 
@@ -36,7 +36,7 @@ sub traverse_array_schemas {
   return E($state, '%s array is empty', $state->{keyword}) if not @{$schema->{$state->{keyword}}};
 
   foreach my $idx (0 .. $#{$schema->{$state->{keyword}}}) {
-    $state->{evaluator}->_traverse($schema->{$state->{keyword}}[$idx],
+    $state->{evaluator}->_traverse_subschema($schema->{$state->{keyword}}[$idx],
       +{ %$state, schema_path => $state->{schema_path}.'/'.$state->{keyword}.'/'.$idx });
   }
 }
@@ -47,7 +47,7 @@ sub traverse_object_schemas {
   return if not assert_keyword_type($state, $schema, 'object');
 
   foreach my $property (sort keys %{$schema->{$state->{keyword}}}) {
-    $state->{evaluator}->_traverse($schema->{$state->{keyword}}{$property},
+    $state->{evaluator}->_traverse_subschema($schema->{$state->{keyword}}{$property},
       +{ %$state, schema_path => jsonp($state->{schema_path}, $state->{keyword}, $property) });
   }
 }
@@ -57,13 +57,13 @@ sub traverse_property_schema {
 
   return if not assert_keyword_type($state, $schema, 'object');
 
-  $state->{evaluator}->_traverse($schema->{$state->{keyword}}{$property},
+  $state->{evaluator}->_traverse_subschema($schema->{$state->{keyword}}{$property},
     +{ %$state, schema_path => jsonp($state->{schema_path}, $state->{keyword}, $property) });
 }
 
 sub eval {
   my ($self, $data, $schema, $state) = @_;
-  $state->{evaluator}->_eval($data, $schema, $state);
+  $state->{evaluator}->_eval_subschema($data, $schema, $state);
 }
 
 1;
@@ -80,7 +80,7 @@ JSON::Schema::Modern::Vocabulary - Base role for JSON Schema vocabulary classes
 
 =head1 VERSION
 
-version 0.512
+version 0.513
 
 =head1 SYNOPSIS
 
@@ -104,7 +104,10 @@ User-defined custom vocabularies are not supported at this time.
 =head2 vocabulary
 
 The canonical URI describing the vocabulary, as described in
-L<JSON Schema Core Meta-specification, section 8.1.2|https://json-schema.org/draft/2019-09/json-schema-core.html#rfc.section.8.1.2>. Must be implemented by the composing class.
+L<JSON Schema Core Meta-specification, section 8.1.2|https://json-schema.org/draft/2019-09/json-schema-core.html#rfc.section.8.1.2>.
+Must be implemented by the composing class. May require the C<specification_version> from the
+L<JSON::Schema::Modern> object (or the C<spec_version> field from the C<$state> object) as an
+argument.
 
 =head2 keywords
 
