@@ -4,7 +4,7 @@ package JSON::Schema::Modern::Vocabulary::MetaData;
 # vim: set ts=8 sts=2 sw=2 tw=100 et :
 # ABSTRACT: Implementation of the JSON Schema Meta-Data vocabulary
 
-our $VERSION = '0.512';
+our $VERSION = '0.513';
 
 use 5.016;
 no if "$]" >= 5.031009, feature => 'indirect';
@@ -17,10 +17,20 @@ use namespace::clean;
 
 with 'JSON::Schema::Modern::Vocabulary';
 
-sub vocabulary { 'https://json-schema.org/draft/2019-09/vocab/meta-data' }
+sub vocabulary {
+  my ($self, $spec_version) = @_;
+  return
+      $spec_version eq 'draft2019-09' ? 'https://json-schema.org/draft/2019-09/vocab/meta-data'
+    : undef;
+}
 
 sub keywords {
-  qw(title description default deprecated readOnly writeOnly examples);
+  my ($self, $spec_version) = @_;
+  return (
+    qw(title description default),
+    $spec_version ne 'draft7' ? 'deprecated' : (),
+    qw(readOnly writeOnly examples),
+  );
 }
 
 sub _traverse_keyword_title {
@@ -75,7 +85,7 @@ JSON::Schema::Modern::Vocabulary::MetaData - Implementation of the JSON Schema M
 
 =head1 VERSION
 
-version 0.512
+version 0.513
 
 =head1 DESCRIPTION
 
